@@ -1,7 +1,7 @@
 package SQL::Interpolate::Filter;
 
 use strict;
-use warnings;
+#use warnings;
 use Filter::Simple;
 use Text::Balanced qw/extract_quotelike
                       extract_bracketed
@@ -9,7 +9,7 @@ use Text::Balanced qw/extract_quotelike
                       extract_variable
                       extract_codeblock/;
 
-our $VERSION = '0.40_03';
+our $VERSION = "0.30";
 
 # Source filter.
 # Note: this could be improved as done in the POD of the development 2.0 version of
@@ -18,22 +18,24 @@ FILTER {
     my $id = qr/\b(?!([ysm]|q[rqxw]?|tr)\b)\w+/;
 
     # This lexes the Perl source code, replacing quotelike sql//
-    # operators with the result of _process_sql().
-    while ($_ !~ /\G$/gc) {
+    # operators with the result of process_sql().
+    while($_ !~ /\G$/gc) {
         my $sql;
         my $last_pos = pos();
-        if    (/\G\s+/gc) { }  # whitespace
-        elsif (/\G#.*/gc) { }  # comments
+        if(/\G\s+/gc) {    # whitespace
+        }
+        elsif(/\G#.*/gc) { # comments
+        }
         # sql// operators
         # FIX:should any other quote delimiters be added?
-        elsif (/\G\bsql\b\s*(?=[\{\(\[\<\/])/gcs &&
+        elsif(/\G\bsql\b\s*(?=[\{\(\[\<\/])/gcs &&
             do {
                 my $pos = pos();
-                s/\G/ q/;  # convert to Perl quote-like
+                s/\G/ q/; # convert to Perl quote-like
                 pos() = $pos;
                 $sql = (extract_quotelike())[5];
                 #print "<sql:$sql>";
-                if (!$sql) {  # restore
+                if(!$sql) { # restore
                     s/\G q//;
                     pos() = $pos;
                 }
@@ -42,19 +44,19 @@ FILTER {
         )
         {
             my $pos = pos();
-            my $out = _process_sql($sql);
+            my $out = &process_sql($sql);
             pos() = $pos;
             substr($_, $last_pos, pos() - $last_pos) = $out;
             pos() = $last_pos + length($out);
         }
         # prevent things like $y = ... = from being interpreted as string.
-        elsif (/\G(?<=[\$\@])\w+/gc) {
+        elsif(/\G(?<=[\$\@])\w+/gc) {
             #print "[DEBUG:var:$&]";
         }
-        elsif (/\G$id/gc) {
+        elsif(/\G$id/gc) {
             #print "[DEBUG:id:$&]";
         }
-        elsif (my $next = (extract_quotelike())[0]) {
+        elsif(my $next = (extract_quotelike())[0]) {
             #print "[DEBUG:q:$next]";
         }
         else {
@@ -66,23 +68,24 @@ FILTER {
 
 # Convert the string inside a sql// quote-like operator into
 # a list of SQL strings and variable references for interpolation.
-sub _process_sql {
+sub process_sql
+{
     local $_ = shift;
 
     my @parts;
     my $instr = 0;
-    while ($_ !~ /\G$/gc) {
+    while($_ !~ /\G$/gc) {
         my $tok;
         my $tok_type;
         my $pos_last = pos();
-        if (/\G(\s+|\*)/gc) {
+        if(/\G(\s+|\*)/gc) {
             $tok = $1;
             $tok_type = 's';
         }
-        elsif ($tok = (extract_variable($_))[0]) {
+        elsif($tok = (extract_variable($_))[0]) {
             $tok_type = 'v';
         }
-        elsif ($tok = (extract_codeblock($_, '{['))[0]) {
+        elsif($tok = (extract_codeblock($_, '{['))[0]) {
             $tok_type = 'c';
         }
         else {
@@ -91,8 +94,8 @@ sub _process_sql {
             $tok_type = 's';
         }
 
-        if ($tok_type eq 's') {
-            if ($instr) {
+        if($tok_type eq 's') {
+            if($instr) {
                 $parts[-1] .= $tok
             }
             else {
@@ -103,27 +106,41 @@ sub _process_sql {
         else {
             $parts[-1] = 'qq[' . $parts[-1] . ']' if $instr;
             $instr = 0;
-            if ($tok_type eq 'v') {
+            if($tok_type eq 'v') {
                 push @parts, '\\' . $tok;
             }
-            elsif ($tok_type eq 'c') {
+            elsif($tok_type eq 'c') {
                 push @parts, $tok;
             }
+            elsif($tok_type eq 'limit') {
+                # note: full package name to not require import
+                push @parts, "SQL::Interpolate::Limit->new$tok";
+            }
             else { die 'assert'; }
         }
 
     }
     $parts[-1] = 'qq[' . $parts[-1] . ']' if $instr;
+    #use Data::Dumper; die "[DEBUG_SQL_PARTS:" . Dumper(\@parts) . "]";
 
-    my $out = 'SQL::Interpolate::Filter::_make_sql('
-            . join(', ', @parts) . ')';
+    my $out = 'SQL::Interpolate::SQL->new(' . join(', ', @parts) . ')';
 
     return $out;
 }
 
+1;
+
+package SQL::Interpolate::SQL;
 # Generated by the sql// operator when source filtering is enabled.
-sub _make_sql {
-    my (@list) = @_;
+
+use strict;
+use base qw(Exporter SQL::Interpolate::Macro);
+use overload '.'  => \&concat,
+             '""' => \&stringify;
+
+sub new
+{
+    my($class, @list) = @_;
 
     # Note that sql[INSERT INTO mytable $x] gets translated to
     #   q[INSERT INTO mytable], \$x
@@ -131,14 +148,42 @@ sub _make_sql {
     # would be difficult to know at source filtering time whether
     # $x is already a reference.  Therefore, we dereference any
     # double reference here (at run-time).
-    do { $_ = $$_ if ref($_) eq 'REF' }
-        for @list;
+    do { $_ = $$_ if ref($_) eq 'REF' } for @list;
 
-    my $o = SQL::Interpolate::SQL->new(@list);
-    return $o;
+    my $self = {
+        list => \@list
+    };
+    bless $self, $class;
+    return $self;
 }
 
-1;
+# implements SQL::Interpolate::Macro
+sub expand
+{
+    my $self = shift;
+    return @{$self->{list}};
+}
+
+# Concatenate SQL object with another expression.
+# An SQL object can be concatenated with another SQL object,
+# variable reference, or an SQL string.
+sub concat
+{
+    my($a, $b, $inverted) = @_;
+
+    my @params = ( @{$a->{list}},
+                   ref($b) eq __PACKAGE__ ? @{$b->{list}} : ($b) );
+    ($params[0], $params[1]) = ($params[1], $params[0]) if $inverted;
+    my $c = new SQL::Interpolate::SQL(@params);
+
+    return $c;
+}
+
+sub stringify
+{
+    my($a) = @_;
+    return $a;
+}
 
 # Implementation Notes:
 # Sub::Quotelike provides similar functionality to this module,
@@ -166,7 +211,6 @@ SQL::Interpolate::Filter - Source filtering for SQL::Interpolate
 
 =head1 SYNOPSIS
 
-  # This first line enables source filtering.
   use SQL::Interpolate FILTER => 1, qw(:all);
   
   ($sql, @bind) = sql_interp sql[
@@ -184,24 +228,21 @@ SQL::Interpolate::Filter - Source filtering for SQL::Interpolate
       }
   ];
 
-  # Each result above is suitable for passing to DBI:
-  my $res = $dbh->selectall_arrayref($sql, undef, @bind);
-
 =head1 DESCRIPTION
 
-This module adds source filtering capability to the
-L<SQL::Interpolate|SQL::Interpolate> and
-L<DBIx::Interpolate|DBIx::Interpolate> modules.  The source filtering
-option provides Perl an additional quote-like operator (see L<perlop>)
-denoted sql//.  The quote can contain SQL and Perl variables:
+This module adds source filtering capability to SQL::Interpolate.
+The source filtering option provides Perl an additional quote-like
+operator (see L<perlop>) denoted sql//.  The quote can contain
+SQL and Perl variables.
 
   sql/SELECT * FROM mytable WHERE x = $x/;
 
-Source filtering will transform this construct into an sql() object
-containing the filtered interpolation list:
+Source filtering will transform this construct into a constructor for
+a macro object that simply contains the filtered interpolation list:
 
-  sql("SELECT * FROM mytable WHERE x = ", \$x);
+  SQL::Interpolate::SQL->new("SELECT * FROM mytable WHERE x = ", \$x);
 
+The macro expands into the contained interpolation list,
 which C<sql_interp> (or C<dbi_interp>) can then interpolate as usual:
 
   "SELECT * FROM mytable WHERE x = ?", ($x)
@@ -211,8 +252,7 @@ which C<sql_interp> (or C<dbi_interp>) can then interpolate as usual:
 To enable the quote-like sql// operator, add a "FILTER => 1" to your use
 statement:
 
-  use SQL::Interpolate  FILTER => 1, qw(:all);  # or
-  use DBIx::Interpolate FILTER => 1, qw(:all);
+  use SQL::Interpolate FILTER => 1, qw(:all);
 
 Just as it is possible to do with q// or qq// operators, you can use
 various delimiters on the sql// operator, such as
@@ -222,8 +262,9 @@ various delimiters on the sql// operator, such as
   sql<SELECT * from mytable WHERE x = $x>
   sql/SELECT * from mytable WHERE x = $x/
 
-sql() objects (and sql// string-like operators representing them) come
-with a string concatenation operator (.), so you can do things like
+SQL:::Interpolate::SQL objects (and sql// operators representing them)
+come with a string concatenation operator (.), so you can
+do things like
 
   sql[
     SELECT partnum, desc, price, stock
@@ -233,32 +274,34 @@ with a string concatenation operator (.), so you can do things like
 =head2 Security notes
 
 An sql// object concatenated with a string will append the string
-verbatim into your result SQL:
+verbatim into your result SQL.  Future versions of SQL::Interpolate
+may throw an error if one attempts to concatenate an sql// object with
+a string.  It's a better idea to do this if you must:
 
-  $dbx->do(sql[UPDATE mytable SET y = 0 WHERE x = ] . $name);  # not good
+  $dbx->do(sql[UPDATE mytable SET y = 0 WHERE x = ] . sql[$name]);
 
-Future versions of SQL::Interpolate may throw an error if one attempts
-to do this.  If you want the value to bind, you must interpolate:
+or simply
 
-  $dbx->do(sql[UPDATE mytable SET y = 0 WHERE x = ] . sql[$name]);  # or
   $dbx->do(sql[UPDATE mytable SET y = 0 WHERE x = $name]);
 
+because then the $name value will be properly interpolated.
+
 =head2 Examples
 
  INPUT:  sql[WHERE one=$x AND $y]
- OUTPUT: sql("WHERE one=", \$x, " AND ", $y)
+ OUTPUT: "WHERE one=", \$x, " AND ", $y
 
  INPUT:  sql[INSERT INTO mytable @x]
- OUTPUT: sql("INSERT INTO mytable ", \@x)
+ OUTPUT: "INSERT INTO mytable ", \@x
 
  INPUT:  sql[INSERT INTO mytable [1, 2]]
- OUTPUT: sql("INSERT INTO mytable ", [1, 2])
+ OUTPUT: "INSERT INTO mytable ", [1, 2]
 
  INPUT   sql[INSERT INTO mytable %x]
- OUTPUT: sql("INSERT INTO mytable ", \%x)
+ OUTPUT: "INSERT INTO mytable ", \%x
 
  INPUT:  sql[INSERT INTO mytable {one => 1, two => 2}]
- OUTPUT: sql("INSERT INTO mytable ", {one => 1, two => 2})
+ OUTPUT: "INSERT INTO mytable ", {one => 1, two => 2}
 
 =head2 Exports and Use Parameters
 
@@ -272,17 +315,10 @@ The source code of the module after source filtering will be sent
 to STDERR.
 
  ...
- SQL::Interpolate::Filter::_make_sql(qq[SELECT * FROM mytable WHERE x = ], \$x)
+ SQL::Interpolate::SQL->new(qq[SELECT * FROM mytable WHERE x = ], \$x)
  ...
 
-=head1 DEPENDENCIES
-
-This module depends on SQL::Interpolate, Filter::Simple (any),
-and Text::Balanced >= 1.87.
-
-=head1 DESIGN NOTES
-
-=head2 Limitations / characteristics
+=head1 LIMITATIONS
 
 Source filtering is somewhat experimental and has the potential to
 give unexpected results because lexing Perl is hard.  The module
@@ -292,7 +328,7 @@ always precise, especially for very obscure Perl constructs.  It
 should work fine though on many things.  If in doubt, check the output
 yourself by enabling the TRACE_FILTER option.
 
-=head2 Proposed enhancements
+=head2 TODO
 
 Support Text::Balanced 2.0 and improved Perl lexing.
 
